#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

# ***************************************************************************
#    copyright            : (C) 2006 by Mathias Monnerville
#    email                : tellico_dev@yahoo.fr
#  	copyright			 : (C) 2008 by Raphal Fischer
#	email			 : fischer.tellico@free.fr
# ***************************************************************************
# You are welcome to report any bug to Raphal Fischer, such as missing or wrong information, when comparing to linked bedetheque.com page.
#
# ***************************************************************************
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of version 2 of the GNU General Public License as  *
# *   published by the Free Software Foundation;                            *
# *                                                                         *
# ***************************************************************************

# $Id: bedetheque.py 123 2008-05-02 raphael $

"""
This script is based on other tellico scripts due to Mathias Monnerville. They were modified and extended to get as much information as possible on the bedetheque.com data base.
You are welcome to report any bug to Raphal Fischer, such as missing or wrong information, when comparing to linked bedetheque.com page.

This script currently provides the following information, corresponding to the bedetheque.com fields :
	
Serie, genre, orgine, langue, note moyenne, synopsis (of the serie)
chronique (if any)
title (of the album), scnariste(s), dessinateur(s), coloriste(s), dpot lgal, achev d'imprim, estimation, editeur, collection, ISBN, planches, info dition
Image de couverture 

This script has to be used with tellico (http://periapsis.org/tellico) as an external data source program.
It allows searching through the bedetheque.com web database.

Related info, critics, and cover are fetched automatically.

Tellico data source setup:
- source name: bedetheque.com (or whatever you want :)
- Collection type: comics collection
- Result type: tellico
- Path: /path/to/script/bedetheque.com.py
- Arguments:
Title (checked) = -t %1
Author (checked) = -a %1
Keyword (checked) = -s %1	
Update (checked) = -t %{title}  (not implemented yet in tellico)

Note : 
	keyword mean serie or cross-search, for instance :
	"donjon" will return every item of the various donjon series (about 50 titles).
	"-s donjon -a andreas" will return only the item of the previous list that Andreas contributed in. (The search is much faster as fewer pages have to be explored)?
	"-a andreas" search as keyword is the same as "andreas" search by author. 
"""

import sys, os, re, md5, random, string
import urllib, urllib2, time, base64
import xml.dom.minidom

XML_HEADER = """<?xml version="1.0" encoding="UTF-8"?>"""
DOCTYPE = """<!DOCTYPE tellico PUBLIC "-//Robby Stephenson/DTD Tellico V9.0//EN" "http://periapsis.org/tellico/dtd/v9/tellico.dtd">"""
NULLSTRING = ''

VERSION = "0.2"
	
	
def genMD5():
	"""
	Generates and returns a random md5 string. Its main purpose is to allow random 
	image file name generation.
	"""
	obj = md5.new()
	float = random.random()
	obj.update(str(float))
	return obj.hexdigest()

class BasicTellicoDOM:
	"""
	This class manages tellico's XML data model (DOM)
	"""
	def __init__(self):
		self.__dict = {"""""" : """&Aacute;""", """""" : """&aacute;""", """""" : """&Acirc;""", """""" : """&acirc;""", """""" : """&Agrave;""", """""" : """&agrave;""", """""" : """&Aring;""",  """""" : """&aring;""",  """""" : """&Atilde;""", """""" : """&atilde;""", """""" : """&Auml;""", """""" : """&auml;""", """""" : """&acute;""", """""" : """&AElig;""", """""" : """&aelig;""", """""" : """&brvbar;""", """""" : """&Ccedil;""", """""" : """&ccedil;""", """""" : """&cedil;""", """""" : """&raquo;""", """""" : """&laquo;""", """""" : """&copy;""", """""" : """&sup2;""", """""" : """&Eacute;""", """""" : """&eacute;""", """""" : """&Ecirc;""", """""" : """&ecirc;""", """""" : """&Egrave;""", """""" : """&egrave;""", """""" : """&Euml;""","""""" : """&euml;""",""" """ : """&nbsp;""","""&""" : """&amp;""","""""" : """&ETH;""","""""" : """&eth;""","""""" : """&iexcl;""", """""" : """&frac34;""", """""" : """&frac12;""", """""" : """&frac14;""",  """""" : """&Iacute;""", """""" : """&iacute;""", """""" : """&Icirc;""",  """""" : """&icirc;""", """""" : """&Igrave;""", """""" : """&igrave;""", """""" : """&Iuml;""", """""" : """&iuml;""", """<""" : """&lt;""", """""" : """&macr;""", """""" : """&reg;""", """""" : """&Ntilde;""", """""" : """&ntilde;""", """""" : """&Oacute;""", """""" : """&oacute;""", """""" : """&Ocirc;""", """""" : """&ocirc;""",  """""" : """&Ograve;""","""""" : """&ograve;""", """""" : """&Oslash;""","""""" : """&oslash;""", """""" : """&Otilde;""", """""" : """&otilde;""", """""" : """&Ouml;""", """""" : """&ouml;""", """""" : """&ordf;""", """""" : """&ordm;""", """""" : """&plusmn;""", """""" : """&iquest;""", """""" : """&middot;""", """""" : """&szlig;""",  """""" : """&deg;""", """""" : """&divide;""", """""" : """&micro;""",  """""" : """&times;""", """""" : """&not;""", """""" : """&para;""", """""" : """&sect;""", """""" : """&yen;""", """>""" : """&gt;""", """""" : """&cent;""", """""" : """&pound;""", """""" : """&curren;""", """""" : """&THORN;""", """""" : """&thorn;""", """""" : """&uml;""", """""" : """&sup3;""", """""" : """&Uacute;""", """""" : """&uacute;""", """""" : """&Ucirc;""",  """""" : """&ucirc;""", """""" : """&Ugrave;""", """""" : """&ugrave;""", """""" : """&Uuml;""", """""" : """&uuml;""","""""" : """&sup1;""","""""" : """&Yacute;""", """""" : """&yacute;""", """""" : """&yuml;"""}

		self.__doc = xml.dom.minidom.Document()
		self.__root = self.__doc.createElement('tellico')
		self.__root.setAttribute('xmlns', 'http://periapsis.org/tellico/')
		self.__root.setAttribute('syntaxVersion', '9')
		
		self.__collection = self.__doc.createElement('collection')
		self.__collection.setAttribute('title', 'My Comics')
		self.__collection.setAttribute('type', '6')

		self.__fields = self.__doc.createElement('fields')
		# Add all default (standard) fields
		self.__dfltField = self.__doc.createElement('field')
		self.__dfltField.setAttribute('name', '_default')
		self.__fields.appendChild(self.__dfltField)
		self.newField(field={'name': 'artist','title': 'Dessinateur','flags': '7','category': 'Gnral','format': '2','type': '1'})
		self.newField(field={'flags' : '7', 'title' : 'Coloriste', 'category' : 'Gnral', 'format': '2', 'type': '1', 'name': 'coloriste'})
		self.newField(field={'flags':'0', 'title':'ISBN', 'category':'Publication', 'format':'4', 'type':'1', 'name':'isbn', 'description':'Numro ISBN'})
		self.newField(field={'flags':'0', 'title':'Estimation', 'category':'Classification', 'format':'4', 'type':'1', 'name':'estimation', 'description':"""Cote de l'exemplaire"""})
		self.newField(field={'flags':'0', 'title':'Dpot lgal', 'category':'Publication', 'format':'4', 'type':'1', 'name':'depotlegal', 'description':"""Dpot lgal"""})
		self.newField(field={'flags':'0', 'title':"""Achev d'imprimer""", 'category':'Publication', 'format':'4', 'type':'1', 'name':'acheveimprimer', 'description':"""Date d'impression"""})		
		self.newField(field={ 'flags':"0", 'title':"""Lien vers bedetheque.com""", 'category':"Gnral", 'format':"4", 'type':"7", 'name':"bdgest"})
		
		field={'name': 'note','title': 'Note','flags': '0','category': 'Personnel','format': '4','type': '14', 'description': """Avis sur l'album"""}
		self.__customField = self.__doc.createElement('field')
		for item, value in field.iteritems():
			self.__customField.setAttribute(item, value)
		minNode = self.__doc.createElement('prop')
		minNode.setAttribute('name', 'minimum')
		minNode.appendChild(self.__doc.createTextNode('1'))
		self.__customField.appendChild(minNode)
		maxNode = self.__doc.createElement('prop')
		maxNode.setAttribute('name', 'maximum')
		maxNode.appendChild(self.__doc.createTextNode('10'))
		self.__customField.appendChild(maxNode)
		self.__fields.appendChild(self.__customField)
		self.__collection.appendChild(self.__fields)

		self.__images = self.__doc.createElement('images')
		self.__root.appendChild(self.__collection)
		self.__doc.appendChild(self.__root)

		# Current movie id. See entry's id attribute in self.addEntry()
		self.__currentId = 0

	def newField(self,field):
		self.__customField = self.__doc.createElement('field')
		for item, value in field.iteritems():
			self.__customField.setAttribute(item, unicode(value, 'latin-1').encode('utf-8'))
		self.__fields.appendChild(self.__customField)
		self.__collection.appendChild(self.__fields)

	def html2utf(self,chaine):
		for vers,de in self.__dict.iteritems():	
			chaine=re.sub(de, vers, chaine)
		chaine=unicode(chaine, 'latin-1').encode('utf-8')
		return chaine
		
	def addEntry(self, albumData):
		"""
		Add a comic entry. 
		Returns an entry node instance
		"""
		d = albumData
		entryNode = self.__doc.createElement('entry')
		entryNode.setAttribute('id', str(self.__currentId))
		fields={	'title':'title',				
				'depotlegal':'depotlegal',
				'country':'country',
				'publisher':'publisher',
				'language':'language',
				'issue':'issue',
				'writer':'writer',
				'artist':'dessin',
				'coloriste':'coloriste',
				'isbn':'isbn',
				'series':'serie',
				'edition':'collec',
				'pages':'pages',
				'language':'language',
				'genre':'genre',
				'note':'note',
				'estimation':'cote',
				'acheveimprimer':'acheveimprimer',
				'bdgest':'bdgest',
				'comments':'comments'}
		s=''
		if len(d['synopsis'])>0:
			s="<b>Synopsis srie : </b>"+d['synopsis']+"\n"
		if len(d['infos'])>0:
			s=s+"<br/><b>Infos dition : </b>"+d['infos']+"\n"
		if d['texte_chronique']:
			s=s+d['texte_chronique']
		d['comments']=s
		if d['image']:
			imageNode = self.__doc.createElement('image')
			imageNode.setAttribute('format', 'JPEG')
			imageNode.setAttribute('id', d['image'][0])
			imageNode.appendChild(self.__doc.createTextNode(unicode(d['image'][1], 'latin-1').encode('utf-8')))
			coverNode = self.__doc.createElement('cover')
			coverNode.appendChild(self.__doc.createTextNode(d['image'][0]))
		for name, value in fields.iteritems():
			Node=self.__doc.createElement(name)
			Node.appendChild(self.__doc.createTextNode(self.html2utf(d[value])))
			entryNode.appendChild(Node)
		if d['image']:
			entryNode.appendChild(coverNode)
			self.__images.appendChild(imageNode)
		self.__collection.appendChild(entryNode)
		self.__currentId += 1
		return entryNode

	def printEntry(self, nEntry):
		"""
		Prints entry's XML content to stdout
		"""
		try:
			print nEntry.toxml()
		except:
			print sys.stderr, "Error while outputing XML content from entry to Tellico"

	def printXMLTree(self):
		"""
		Outputs XML content to stdout
		"""
		self.__collection.appendChild(self.__images)
		print XML_HEADER; print DOCTYPE
		print self.__root.toxml()


class BDthequeParser:	
	def __init__(self):
		self.__baseURL 	 = 'http://www.bedetheque.com'
		self.__basePath  = '/'
		self.__searchserieURL = '/list_series.php?FiltreSerie=%s'
		self.__searchauteurURL = '/list_auteurs.php?FiltreAuteur=%s'
		self.__coverPath = 'thb_couv/'
		self.__SerieURL  = self.__baseURL + self.__basePath

		# Define some regexps
		self.speciaux=['^', '.', '[', ']', '$', '(', ')', '*', '+', '?', '|', '{', '}', '\\']
		
		self.__dict={'[]' : 'a',
				'[]' : 'ae',
				'[]' : 'c',
				'[]' : 'e',
				'[]' : 'd',
				'[]' : 'o',
				'[]' : 'i',
				'[]' : 'r',
				'[]' : 'n',
				'[]' : 'u',
				'[]' : 'y'}
				
		self.__dict2={'<[^>]*>' : '', '\t': '', '<br [^>]*>' : '\n'}
			
		self.__regExps = { 	'title' 	: '<b>[^<]*<font [^>]*>[\. ]{0,2}([^><]+)</b></FONT>',
						'issue'	: '<b>([0-9]*)<font [^>]*>[. ]{0,2}[^><]+</b></FONT>',
						'depotlegal': 'Dpot lgal :</td><td>([^<]*)',
						'acheveimprimer': 'Achev impr. :</td><td>([^<]*)',
						'dessin'	: '<tr><td>Dessin :</td><td><A HREF="[^>]+>([^<]*)</A></td></tr>',
						'writer'	: '<tr><td>Scnario :</td><td><A HREF=[^>]+>([^<]+)</A></td></tr>',
						'coloriste': '<tr><td>Couleurs :</td><td><A [^>]+>([^<]*)</A></td></tr>',
						'publisher'	: '<tr><td>Editeur :</td><td>([^<]*)</td></tr>',
						'collec'	: '<tr><td>Collection : </td><td>([^<]*)</td></tr>',
						'isbn'	: '<tr><td>ISBN :</td><td>([^<]*)</td></tr>',
						'infos'	: '<b>Info dition : </b><i>([^<]*)</i></div>',
						'cote'	: '<tr><td>Estimation :</td><td>([^<]*)</td></tr>',
						'pages' 	: '<tr><td>Planches :</td><td>([0-9]*)</td></tr>',
						}	
						
						
		self.__regexp_alt={'title'	: '<tr><td><a[^>]*><FONT [^>]*><b>([^<]*)</b></FONT>',
						'dessin'	: """<tr><td>Dessin :</td><td><A HREF="[^>]*><font [^>]*>([^<]*)</font></A></td></tr>"""}
		self.__regexp_serie={'genre' 	: '<FONT FACE="Verdana" SIZE=2 COLOR="#294A6B"><i>([^<]*)</i></font><br>',
						'country'	: '<FONT COLOR="#990000">[^<]*</font><br>[^<]*Identifiant BEL : [^<]*<br>[^<]*Origine : ([^<]*)<br>[^<]*Langue : [^<]*<br>[^<]*',
						'language'	: '<FONT COLOR="#990000">[^<]*</font><br>[^<]*Identifiant BEL : [^<]*<br>[^<]*Origine : [^<]*<br>[^<]*Langue : ([^<]*)<br>[^<]*',
						'note'	: 'Note moyenne de la srie : <IMG SRC="images/note([0-9]*).gif" border=0 ALT="[^<]*"><br>',
						'synopsis': 'Note moyenne de la srie : <IMG SRC="images/note[0-9].gif" border=0 ALT="[^<]*"><br>[^<]*<div align="justify"><BR>([^<]*)<',
						'tout':'<IMG SRC="(images)/Toutoff.gif" BORDER=0 WIDTH=39 HEIGHT=19>'}
						
						#	<b>Chronique BD Gest'</b>&nbsp;&nbsp;<A HREF="http://www.bdgest.com/critique_898.html" title="Voir la critique" target="_blank"><font color="#CC0000">[Donjon Crpuscule 104. Le Dojo du Lagon]</font></a><br><img src="images/spacer_h.gif" width=5 heig						
						
		self.__chronique={'lien' 	: """<b>Chronique BD Gest'</b>&nbsp;&nbsp;<A HREF="(http://www.bdgest\.com/[^.]*\.html)" title="Voir la critique" target="_blank"><font color="#CC0000">([^<]*)</font>""",
						'texte' 	: """<A HREF="critique\.php\?a=[0-9]*"><i>(.*)"""}
#<a href="critique.php?a=173"><i>Par L. Cirade</i></a>&nbsp;
		self.__serieRE = {
						'inner' :	'<table width="100%" cellpadding="0" cellspacing="0">\s*<tr>\s*<td>\s*<IMG SRC="images/spacer_h.gif" BORDER=0 WIDTH=0 HEIGHT=5><BR>\s*<TABLE WIDTH="100%" BORDER=0  CELLPADDING=0 CELLSPACING=0>(?P<page>.*)<table><tr><td height="5"></td></tr></table>\s*</td></tr></table>\s*<IMG SRC="images/spacer_h.gif" BORDER=0 WIDTH=0 HEIGHT=5>',
						'item' : '<td align="center" valign="top" WIDTH="150"><table[^>]*>.*?Vous devez vous <A HREF="index.php"><u>identifier</u></a> pour saisir un commentaire, une correction ou voter pour cet album.<br></font>',
						'split' : '<td align[^>]*><table[^>]*>'}	


		# Compile patterns objects
		self.__chronPO = {}
		for k, pattern in self.__chronique.iteritems():
			self.__chronPO[k]= re.compile(pattern,18)
		self.__regExpsPO = {}
		for k, pattern in self.__regExps.iteritems():
			self.__regExpsPO[k] = re.compile(pattern,18)
		
		self.__seriePO = {}
		for k, pattern in self.__regexp_serie.iteritems():
			self.__seriePO[k] = re.compile(pattern,18)

		self.__regExps_altPO = {}
		for k, pattern in self.__regexp_alt.iteritems():
			self.__regExps_altPO[k] = re.compile(pattern,18)

		self.__count = 0
		self.__domTree = BasicTellicoDOM()


	def __fetchCover(self, path, delete = True):
		"""
		Fetch cover to /tmp. Returns base64 encoding of data.
		The image is deleted if delete is True
		"""
		md5 = genMD5()
		imObj = urllib2.urlopen(path.strip())
		img = imObj.read()
		imObj.close()
		imgPath = "/tmp/%s.jpeg" % md5
		try:
			f = open(imgPath, 'w')
			f.write(img)
			f.close()
		except:
			print sys.stderr, "Error: could not write image into /tmp"

		b64data = (md5 + '.jpeg', base64.encodestring(img))

		# Delete temporary image
		if delete:
			try:
				os.remove(imgPath)
			except:
				print sys.stderr, "Error: could not delete temporary image /tmp/%s.jpeg" % md5
		return b64data

	def __fetchAlbumInfo(self,page):
		"""
		Looks for movie information
		"""
		# First grab picture data
		#imgMatch = re.search("""<span [^>]*><A HREF="javascript:afficheMaxi\('Couvertures[^>]*><IMG SRC="%s([^"]*)""" % self.__coverPath, page,18)
	#</font></A></td></tr><tr><td></td><td><A HREF="http://www.bedetheque.com/auteur-14460-BD-Doo-Robin.html" target="_top" alt="Voir la fiche de Doo, Robin" title="Voir la fiche de Doo, Robin">
		page=re.sub("""</a></td></tr><tr><td></td><td><a [^>]*>(?i)""",""";""",page)
		page=re.sub("""</font>""","",page)
		#page=re.sub("""Sfar""","Sphare",page)	
		
		imgMatch = re.search("""<IMG SRC="%s([^"]*)""" % self.__coverPath, page,18)
		if imgMatch:
			imgPath = self.__SerieURL + self.__coverPath + imgMatch.group(1)
			# Fetch cover and gets its base64 encoded data
			b64img = self.__fetchCover(imgPath)
		else:
			b64img = None
		matches = {}
		data = {}
		# Default values
		data['image'] 		= b64img
		data['depotlegal']	= NULLSTRING
		for name, po in self.__regExpsPO.iteritems():
			data[name] = NULLSTRING
			matches[name] = po.findall(page)
			if matches[name]:
				for item in matches[name]:
					if len(data[name])>0:
						data[name] += ";"+item
						#print item
					else:
						data[name] = item
						#print item
		for name, po in self.__regExps_altPO.iteritems():
			matches[name] = po.findall(page)
			if matches[name]:
				for item in matches[name]:
					if len(data[name])>0:
						data[name] += ";"+item
						#print item
					else:
						data[name] = item
						#print item
		return data

	def __fetchSerieInfo(self):
		#print  "Looks for serie information"
		matches = {}
		data = {}
		# Default values
		for name, po in self.__seriePO.iteritems():
			data[name] = NULLSTRING
			matches[name] = po.search(re.sub('<br />','!br!',self.__data))
			if matches[name]:
				data[name] = re.sub('!br!','<br />',matches[name].group(1))
		# On note les chroniques s'il y en a
		data['chroniques']=re.findall(self.__chronique['lien'],self.__data)
		#print data
		return data
				


	def __fetchMovieLinks(self,motif):
		"""
		Retrieve all links related to the search. self.__data contains HTML content fetched by self.__getHTMLContent() 
		that need to be parsed.
		"""
		matchList = re.findall(motif, self.__data,18)
		#print matchList
		if not matchList: return None
		return matchList

	def __getMovie(self):
		# On commence par faire une liste des sries  traiter, que l'on cherche par auteur ou titre
		self.__count2=0
		self.__countt,self.__counta,self.__counts=0,0,0
		links_a,links_s,links_t=[],[],[]
		if len(self.__author)>0:
			#print "Je cherche "+auteur
			self.__getHTMLContent("%s%s" % (self.__baseURL, self.__searchauteurURL % urllib.quote(self.__author)))
			links_auteur = []
			links1=self.__fetchMovieLinks("""<a HREF="%s/([^"]*?)"[^>]*>([^<]*)</a>""" % self.__baseURL)  
			links2=self.__fetchMovieLinks("""<a HREF="%s/([^"]*?)"[^>]*><Font[^>]*>([^<]*)</font></a>""" % self.__baseURL)
			if links1:
				links_auteur=links1
			if links2:
				links_auteur+=links2
			# On fait la liste des auteurs correspondants
			# Puis la liste des sries de chaque auteur
			if links_auteur:
				for link in links_auteur:
					url,nom=link[0],re.sub("<[^>]*>",'',link[1])
					if self.IsNameIn(self.__author,nom):
						self.__getHTMLContent("%s%s" % (self.__SerieURL, url))
						otherlinks=self.__fetchMovieLinks("""<A HREF="/(serie-[^"]*)">([^<]*)</A>""")
						if otherlinks:
							links_a = links_a+otherlinks 
			if not links_a:
				return None
			
		if len(self.__serie)>0: 
			for de, vers in self.__dict.iteritems():	
				self.__serie=re.sub(de, vers, self.__serie)
			self.__getHTMLContent("%s%s" % (self.__baseURL, self.__searchserieURL % urllib.quote(self.__serie)))
			# Get all links
			links_s = self.__fetchMovieLinks("""<a HREF="%s/(?P<page>.*?)"[^>]*>(?P<title>.*?)</a>""" % self.__baseURL)
			if not links_s:
				return None

		if len(self.__title)>0 and not self.__title==self.__serie: 
			for de, vers in self.__dict.iteritems():	
				self.__title=re.sub(de, vers, self.__title)
			self.__getHTMLContent("%s%s" % (self.__baseURL, self.__searchserieURL % urllib.quote(self.__title)))
			# Get all links
			links_t = self.__fetchMovieLinks("""<a HREF="%s/(?P<page>.*?)"[^>]*>(?P<title>.*?)</a>""" % self.__baseURL)	
			if not links_t:
				return None
		
		# Croisement des critres :
		if links_t and not self.__serie==self.__title:
			if len(links_t)>0:
				links=links_t
				self.__countt=len(links_t)
		else:
			self.__countt=0
		if links_s:
			if len(links_s)>0:
				links=links_s
				self.__counts=len(links_s)
		else:
			self.__counts=0
		if links_a:
			if len(links_a)>0:
				links=links_a
				self.__counta=len(links_a)
		else:
			self.__counta=0
		links0={}
		for entry in links:
			ok=1
			if self.__counts>0:
				ok=0
				for verif in links_s:
					if verif[0]==entry[0]:
						ok=1
			if self.__countt>0:
				ok=0
				for verif in links_t:
					if verif[0]==entry[0]:
						ok=1
			if ok==1:
				links0[entry[0]]=entry[1]
		links=links0
		# On a selectionn les sries, on recherche les albums :
		if links:
			for url, serie in links.iteritems():
				# On vrifie que le nom de la srie correspond au critre "serie"
				# Recherche des infos sur la srie
				self.__getHTMLContent(self.__SerieURL + re.sub('.html','__10000.html', url))
				#self.__getHTMLContent(self.__SerieURL +url)
				data_serie = self.__fetchSerieInfo()
				## Si la srie a plusieurs pages, il faut aller chercher la page complte :
				#if data_serie['tout']:
				
				serie=re.search('<h1>([^<]*)</h1>',self.__data,16).group(1)
				data_serie['serie']=serie
				data_serie['bdgest'] = "%s%s" % (self.__SerieURL, re.sub('.html','__10000.html', url))
				# Dans la page de la srie, on spare les donnes de chaque album
				matchList=re.findall(self.__serieRE['item'],self.__data,16)
				
				for item in matchList:
					# Recherche des infos sur l'album
					data = self.__fetchAlbumInfo(item)
					#print data
					# Si la srie correspond  la recherche, on note tous les albums, sinon, seulement ceux qui correspondent eux-mme  la recherche. (Pas envie de se payer tous les mickey parcequ'il y en a un qui correspond).
					if self.IsTitleIn(self.slash(self.__title),serie+" "+data['title']):
						for label, value in data_serie.iteritems():
							data[label]=value
						# On vrifie que l'auteur cherch fait bien partie des noms cits
						isquoted=1
						if len(self.__author)>0:
							isquoted=0
							for label in ['dessin', 'writer', 'coloriste', 'infos']:
								value=data[label]
								if self.IsAuteurIn(value):
									isquoted=1
						if isquoted==1:
							# On va chercher les chroniques ventuelles
							data['texte_chronique']=''
							if data['chroniques']: # Y'a t'il des chroniques ?
								for chro in data['chroniques']:
									url,nom=chro[0], chro[1]
									#print data['title'],nom
									if re.match(".*"+self.slash(data['title']),nom,18): # Y'en a t'il une pour cet album ?
										data['texte_chronique']=self.__getcritique(url)
										#print data['texte_chronique']
							node = self.__domTree.addEntry(data)
							self.__count+=1
						# Print entries on-the-fly
						#self.__domTree.printEntry(node)
		else:
			return None
		
	def __getcritique(self, url):
		u = urllib2.urlopen(url)
		critique = u.read()
		u.close()
		chro2=re.search(self.__chronique['texte'],critique,18).group(1).split("Votre avis nous intresse")[0]
		chro2=self.html2txt(chro2)
		#print chro2
		return "<br/><b>Chronique : </b>%s" % chro2
	
	def slash(self, expr):
		for de in self.speciaux:
			expr=re.sub('\\'+de,'\\'+de,expr)
		return expr

	def IsNameIn(self,search,str1):
		if len(str1)==0:
			return 0==1
		if len(search)==0:
			return 1==1
		for de, vers in self.__dict.iteritems():	
			str1=re.sub(de, vers, str1)
			search=re.sub(de, vers, search)
		ok=1
		str1=re.sub('-',' ',str1)
		mots1=re.sub(',','',str1).split(' ')		
		mots2=search.split(' ')	
		for mot2 in mots2:
			ok2=0
			for mot1 in mots1:
				if re.match(".*"+mot1,mot2,18):
					ok2=1
			ok*=ok2
		return ok==1
		
	def IsAuteurIn(self,str1):
		if len(str1)==0:
			return 0==1
		for de, vers in self.__dict.iteritems():	
			str1=re.sub(de, vers, str1)
		mots2=self.__author.split(' ')
		ok=1 
		for mot2 in mots2:
			if not re.match(".*"+mot2,str1,18):
				ok=0
		return ok==1


	def IsTitleIn(self,title,str2):
		#str2=unicode(str2, 'latin-1').encode('utf-8')
		#print "%s %s" % (title,str2)
		for de, vers in self.__dict.iteritems():	
			str2=re.sub(de, vers, str2)
		return re.match(".*"+title,str2,18)

	def html2txt(self,chaine):
		for de, vers in self.__dict2.iteritems():	
			chaine=re.sub(de, vers, chaine)
		return chaine


	def __getHTMLContent(self, url):
		"""	Fetch HTML data from url	"""
		#print url
		u = urllib2.urlopen(url)
		self.__data = u.read()
		u.close()
		self.__count2+=1
	
	def run(self,author,serie,title):
		self.__author, self.__serie,self.__title=author,serie,title
		self.__getMovie()
		# Print results to stdout
		self.__domTree.printXMLTree()
		print unicode("<!--\tRecherche de :\nSerie : '%s' (%s sries trouves)\nTitre : '%s' (%s sries trouves)\nAuteur : '%s' (%s sries trouves)\n %s albums trouvs, et %s pages explores-->" % (self.__serie, self.__counts, self.__title, self.__countt, self.__author,self.__counta, self.__count, self.__count2), 'latin-1').encode('utf-8')

	def maj(self,argu):
		self.__lien,self.__isbn,self.__title,self.__date,self.__serie=argu[0],argu[1],argu[2],argu[3],argu[4]
		#self.__lien,self.__isbn,self.__title,self.__date,self.__serie=argu[2],argu[3],argu[4],argu[5],argu[6]
		#self.__lien,self.__isbn,self.__title,self.__date,self.__serie=argu[2],'','','',''

		self.__count2=0
		if len(self.__lien)==0: # L on est mal barr, il y a mme pas le lien. L'ISBN ne doit pas y tre non plus
			print """<!-- Pff... je veux bien essayer, mais il manque le lien-->"""
			run('',self.__serie, self.__title)
			return None
		self.__getHTMLContent(re.sub('__10000.html', '.html', self.__lien))
		data_serie = self.__fetchSerieInfo()
		# Si la srie a plusieurs pages, il faut aller chercher la page complte :
		if data_serie['tout']:
			self.__getHTMLContent(self.__lien)
		serie=re.search('<h1>([^<]*)</h1>',self.__data,16).group(1)
		data_serie['serie']=serie
		data_serie['bdgest'] = self.__lien
		
		matchList=re.findall(self.__serieRE['item'],self.__data,16)
		for item in matchList:
			data = self.__fetchAlbumInfo(item)
			if (data['title']==self.__title or len(self.__title)==0):
				# C'est dj pas mal. Si on a pas mieux, on prend a.
				for label, value in data_serie.iteritems():
					data[label]=value
				# On va chercher les chroniques ventuelles
				data['texte_chronique']=''
				if data['chroniques']: # Y'a t'il des chroniques ?
					for chro in data['chroniques']:
						url,nom=chro[0], chro[1]
						#print data['title'],nom
						if re.match(".*"+self.slash(data['title']),nom,18): # Y'en a t'il une pour cet album ?
							data['texte_chronique']=self.__getcritique(url)
							#print data['texte_chronique']
				data_aupire=data
				if (data['isbn']==self.__isbn or len(self.__isbn)==0) and (data['title']==self.__title or len(self.__title)==0) and (data['depotlegal']==self.__date or len(self.__date)==0):
					node = self.__domTree.addEntry(data)
					self.__domTree.printXMLTree()
					return None
		# On a pas trouv, on fait avec ce qu'on a
		print """<!-- J'ai pas trouv pile le bon album, mais on va faire avec celui l. Le titre au moins est bon-->"""
		node = self.__domTree.addEntry(data_aupire)
		self.__domTree.printXMLTree()
		

def halt():
	#print "HALT."
	sys.exit(0)

def showUsage():
	print """Usage: options 
Options : 
	-t serie or album title 
	-a author
	-s serie or mixed search
keyword is a string that can contain both title and author, or serie :
Exemples : 
	-s '-t title -a author'
	-s '-s serie -t title
The use of the -s argument is a sensible improvement in speed, for 500-long series are no longer analysed if the serie's title does not match.
If you only use the serie argument, you can type :
	-s serie"""
		
	sys.exit(1)

def main():
	
	if len(sys.argv) < 2:
		showUsage()
		
	parser = BDthequeParser()
	
	opts =  {'-t' : 'title', '-a' : 'author', '-s' : 'serie or mixed search', '-m' : 'mise  jour'}
	if sys.argv[1] not in opts.keys():
		showUsage()

	author,serie,title='','',''
	if sys.argv[1] == '-s':
		args=(' '+sys.argv[2]).split(' -')
		if len(args)==1:
			serie=sys.argv[2]	
		else:
			for arg in args:
				if len(arg)>2:
					argu=re.sub(' *$','',arg[2:])
					argu=re.sub('^ *','',argu)
					if arg[0]=='t':
						title=argu
					if arg[0]=='a':
						author=argu
					if arg[0]=='s':
						serie=argu	
	elif sys.argv[1]=='-m':
		argu=sys.argv[2].split(';') 
		#argu=sys.argv
		parser.maj(argu)	
	elif sys.argv[1]=='-t':
		title=sys.argv[2]
	elif sys.argv[1]=='-a':
		author=sys.argv[2]
	#print author
	if len(title)==0:
		title=serie
	if len(author)+len(serie)+len(title)>0:
		parser.run(author,serie,title)
	print """<!-- arguments : %s  -->""" % sys.argv
	
if __name__ == '__main__':
	main()
